@@ -106,7 +106,7 @@ def register_dev(options, dev):
 	return not bool(run_cmd(options, cmd)["err"])
 
 def reserve_dev(options, dev):
-	cmd = options["--mpathpersist-path"] + " -o --reserv --prout-type=5 --param-rk=" + options["--plug"] + " -d " + dev
+	cmd = options["--mpathpersist-path"] + " -o --reserve --prout-type=5 --param-rk=" + options["--plug"] + " -d " + dev
 	return not bool(run_cmd(options, cmd)["err"])
 
 def get_reservation_key(options, dev):
@@ -297,7 +297,7 @@ longer be able to write to the device(s). A manual reboot is required."
 	if not ("--devices" in options and options["--devices"]):
 		fail_usage("Failed: No devices found")
 
-	options["devices"] = options["--devices"].split(",")
+	options["devices"] = [d for d in re.split("\s*,\s*|\s+", options["--devices"].strip()) if d]
 	# Input control END
 
 	result = fence_action(None, options, set_status, get_status)
